@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -27,8 +27,12 @@
 #include <glib-object.h>
 #include <dbus/dbus-glib-bindings.h>
 
+#include <libtracker-common/tracker-common.h>
+
 #include <libtracker-data/tracker-data-manager.h>
-#include <libtracker-common/tracker-dbus.h>
+
+#include <libtracker-client/tracker-sparql-builder.h>
+#include <libtracker-client/tracker.h>
 
 #include <tracker-store/tracker-push-registrar.h>
 #include <tracker-store/tracker-store.h>
@@ -404,6 +408,7 @@ tracker_kmail_registrar_set (TrackerKMailRegistrar *object,
 	request_id = tracker_dbus_get_next_request_id ();
 
 	tracker_dbus_request_new (request_id,
+	                          context,
 	                          "D-Bus request to set one: 'KMail' ");
 
 	dbus_async_return_if_fail (subject != NULL, context);
@@ -420,9 +425,8 @@ tracker_kmail_registrar_set (TrackerKMailRegistrar *object,
 	                            GUINT_TO_POINTER (modseq),
 	                            NULL);
 
+	tracker_dbus_request_success (request_id, context);
 	dbus_g_method_return (context);
-
-	tracker_dbus_request_success (request_id);
 }
 
 void
@@ -445,12 +449,14 @@ tracker_kmail_registrar_set_many (TrackerKMailRegistrar *object,
 
 	len = g_strv_length (subjects);
 
-	dbus_async_return_if_fail (len == predicates->len, context);
-	dbus_async_return_if_fail (len == values->len, context);
+	dbus_async_return_if_fail (predicates->len == len, context);
+	dbus_async_return_if_fail (values->len == len, context);
 
 	tracker_dbus_request_new (request_id,
-	                          "D-Bus request to set many: 'KMail' "
-	                          "'%d'", len);
+	                          context,
+	                          "%s(len:%d)",
+	                          __FUNCTION__,
+	                          len);
 
 	while (subjects[i] != NULL) {
 		perform_set (object,
@@ -464,9 +470,8 @@ tracker_kmail_registrar_set_many (TrackerKMailRegistrar *object,
 	                            GUINT_TO_POINTER (modseq),
 	                            NULL);
 
+	tracker_dbus_request_success (request_id, context);
 	dbus_g_method_return (context);
-
-	tracker_dbus_request_success (request_id);
 }
 
 
@@ -482,12 +487,14 @@ tracker_kmail_registrar_unset_many (TrackerKMailRegistrar *object,
 
 	request_id = tracker_dbus_get_next_request_id ();
 
-	tracker_dbus_request_new (request_id,
-	                          "D-Bus request to unset many: 'KMail' "
-	                          "'%d'", g_strv_length (subjects));
-
 	dbus_async_return_if_fail (subjects != NULL, context);
 
+	tracker_dbus_request_new (request_id,
+	                          context,
+	                          "%s(len:%d)",
+	                          __FUNCTION__,
+	                          g_strv_length (subjects));
+
 	while (subjects[i] != NULL) {
 		perform_unset (object, subjects[i]);
 		i++;
@@ -495,9 +502,8 @@ tracker_kmail_registrar_unset_many (TrackerKMailRegistrar *object,
 
 	tracker_store_queue_commit (on_commit, NULL, GUINT_TO_POINTER (modseq), NULL);
 
+	tracker_dbus_request_success (request_id, context);
 	dbus_g_method_return (context);
-
-	tracker_dbus_request_success (request_id);
 }
 
 void
@@ -511,18 +517,19 @@ tracker_kmail_registrar_unset (TrackerKMailRegistrar *object,
 
 	request_id = tracker_dbus_get_next_request_id ();
 
-	tracker_dbus_request_new (request_id,
-	                          "D-Bus request to unset one: 'KMail'");
-
 	dbus_async_return_if_fail (subject != NULL, context);
 
+	tracker_dbus_request_new (request_id,
+	                          context,
+	                          "%s()",
+	                          __FUNCTION__);
+
 	perform_unset (object, subject);
 
 	tracker_store_queue_commit (on_commit, NULL, GUINT_TO_POINTER (modseq), NULL);
 
+	tracker_dbus_request_success (request_id, context);
 	dbus_g_method_return (context);
-
-	tracker_dbus_request_success (request_id);
 }
 
 void
@@ -536,7 +543,9 @@ tracker_kmail_registrar_cleanup (TrackerKMailRegistrar *object,
 	request_id = tracker_dbus_get_next_request_id ();
 
 	tracker_dbus_request_new (request_id,
-	                          "D-Bus request to cleanup: 'KMail'");
+	                          context,
+	                          "%s()",
+	                          __FUNCTION__);
 
 	perform_cleanup (object);
 
@@ -544,9 +553,8 @@ tracker_kmail_registrar_cleanup (TrackerKMailRegistrar *object,
 	                            GUINT_TO_POINTER (modseq),
 	                            NULL);
 
+	tracker_dbus_request_success (request_id, context);
 	dbus_g_method_return (context);
-
-	tracker_dbus_request_success (request_id);
 }
 
 
