@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -22,7 +22,7 @@
 
 #include "config.h"
 
-#include <libtracker-common/tracker-ontologies.h>
+#include <libtracker-data/tracker-data.h>
 
 #include "tracker-events.h"
 
